/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

#ifndef _NADAR_CONFIGURE_H_INCLUDED_
#define _NADAR_CONFIGURE_H_INCLUDED_

/*****************************************************************************/
/* Nadar's Version.                                                          */
/*****************************************************************************/

#ifndef VERSION
#define VERSION "Nadar-unknown-version"
#endif

/*****************************************************************************/
/* Definitions of Parameters.                                                */
/*****************************************************************************/

/*===========================================================================*/
/* You can change below parameters as you like.                              */
/*===========================================================================*/

#define PORT 12010

/* Default map size is 40x40 */
#define MAP_WIDTH  32
#define MAP_HEIGHT 32

/* If you want to play as a peephole game, define under line. */
#if 0
#define PEEPING
#endif

#ifdef PEEPING
#define GAME_WINDOW_WIDTH  25 /* Default peephole size is 25x25. */
#define GAME_WINDOW_HEIGHT 25
#else
#define GAME_WINDOW_WIDTH  MAP_WIDTH
#define GAME_WINDOW_HEIGHT MAP_HEIGHT
#endif

/* Specify the default size of display by percent. */
#ifdef PEEPING
#define DISPLAY_DEFAULT_SIZE 100 /* Default size is 100% */
#else
#define DISPLAY_DEFAULT_SIZE 50 /* Default size is 50% */
#endif

#define PLAYER_NUMBER_MIN 1 /* Game will start with 1 player. */
#define CANON_NUMBER 3    /* Default number of canon. */
#define MISSILE_NUMBER 3  /* Default number of missiles. */
#define TANK_SPEED 0.3
#define CANON_SPEED 0.8
#define CANON_OFFSET 0.6
#define MISSILE_SPEED 0.8
#define MISSILE_QUICK 0.18
#define MISSILE_OFFSET 0.6
#if 0
#define CANON_INTERVAL 10
#else
#define CANON_INTERVAL 18
#endif
#define MISSILE_INTERVAL 10
#define TANK_DESTROY_INTERVAL 25

/*===========================================================================*/
/* Don't change below parameters.                                            */
/*===========================================================================*/

/* Font size. */
#define FONT_WIDTH   8
#define FONT_HEIGHT 16
#define FONT_OFFSET 14
#define TEXT_WINDOW_WIDTH 20 /* Text window's width is 20 characters. */

#define ICON_SIZE 48      /* Icon size is 48x48. */
#define CHARACTER_SIZE 32 /* The size of map characters is 32x32. */
#define TANK_SIZE 48      /* Tank size is 48x48. */
#define CANON_SIZE 10     /* Canon size is 10x10. */
#define MISSILE_SIZE 10   /* Missile size is 10x10. */
#define EXPLOSION_SIZE 48 /* Explosion size is 48x48. */
#define TANK_ANGLE_NUMBER 24
#define MAP_PIXMAP_NUMBER 16
#define MAP_CITY_MIN 1
#define MAP_CITY_MAX 10
#define MAP_BRICK_MIN 1
#define MAP_BRICK_MAX 3
#define MAP_FOREST_MIN 11
#define MAP_FOREST_MAX 15
#define DISPLAY_INTERVAL 25 /* time interval in milliseconds */
#define CLIENT_BUFFER_SIZE 300

/* С饤ȴ֤Υޥ */
#define JOIN_REQUEST   "JoinRequest"
#define JOIN_ACCEPT    "JoinAccept"
#define DELETE_ME      "DelMe"
#define DELETE_YOU     "DelYou"
#define DISPLAY_CLEAR  "DL"
#define DISPLAY_COPY   "DC"
#define SEND_KEY       "SK"
#define READ_KEY       "RK"
#define DUMMY          "DM" /* ѥåѥߡ */
#define TEST           "Test"

#define PLAYER_DATA    "Player"

#define MAP_DRAW       "MD"
#define TANK_DRAW      "TD"
#define CANON_DRAW     "CD"
#define MISSILE_DRAW   "MS"
#define EXPLOSION_DRAW "ED"
#define GAME_CLEAN     "GC"
#define TEXT_CLEAR     "TextClear"
#define TEXT_DRAW      "TextDraw"

#endif /* _NADAR_CONFIGURE_H_INCLUDED_ */

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
