/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* Missile 饹(ߥѤΥ饹)Υ饤֥                  */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "MissileP.h"

/*****************************************************************************/
/* ؿ                                                                */
/*****************************************************************************/

/*===========================================================================*/
/* ѿμ                                                          */
/*===========================================================================*/

double MissileGetX(Missile missile) { return (missile->x); }
double MissileGetY(Missile missile) { return (missile->y); }
int MissileGetState(Missile missile)        { return (missile->state); }
int MissileSetState(Missile missile, int s) { return (missile->state = s); }
Tank MissileGetTank(Missile missile) { return (missile->tank); }

/*===========================================================================*/
/* ȯȽ                                                                */
/*===========================================================================*/

int MissileCheckExplosion(Missile missile, Map map, SubstanceList tank_list,
			  SubstanceList explosion_list)
{
  Tank tank;
  Explosion explosion;
  double x, y, d, size;

  if (MapGetState(map,
		  ConvertXFromPositionToMap(missile->x),
		  ConvertYFromPositionToMap(missile->y))) {
    MapChangeState(map,
		   ConvertXFromPositionToMap(missile->x),
		   ConvertYFromPositionToMap(missile->y), 0);
    missile->state = MISSILE_EXPLOSION;
    return (missile->state);
  }

  size = (double)EXPLOSION_SIZE / CHARACTER_SIZE * 0.5 * 1.2;

  for (SubstanceListSetCurrentToStart(tank_list);
       !SubstanceListIsCurrentEndEdge(tank_list);
       SubstanceListMoveCurrentToNext(tank_list)) {
    tank = (Tank)SubstanceListGetSubstance(tank_list);
    x = missile->x - TankGetX(tank);
    y = missile->y - TankGetY(tank);
    d = x * x + y * y;
    if (d < size * size) {
      missile->state = MISSILE_EXPLOSION;
      return (missile->state);
    }
  }

  for (SubstanceListSetCurrentToStart(explosion_list);
       !SubstanceListIsCurrentEndEdge(explosion_list);
       SubstanceListMoveCurrentToNext(explosion_list)) {
    explosion = (Explosion)SubstanceListGetSubstance(explosion_list);
    if (ExplosionCheck(explosion, missile->x, missile->y)) {
      missile->state = MISSILE_EXPLOSION;
      return (missile->state);
    }
  }

  return (missile->state);
}

/*===========================================================================*/
/* åȤθ                                                          */
/*===========================================================================*/

static Tank MissileSearchTarget(Missile missile, SubstanceList tank_list)
{
  double dx, dy, d = -1.0;
  Tank tank, target;

  target = NULL;
  for (SubstanceListSetCurrentToStart(tank_list);
       !SubstanceListIsCurrentEndEdge(tank_list);
       SubstanceListMoveCurrentToNext(tank_list)) {
    tank = (Tank)SubstanceListGetSubstance(tank_list);
    if (tank == missile->tank) continue;
    if (TankGetState(tank) == TANK_DESTROY) continue;
    dx = TankGetX(tank) - missile->x;
    dy = TankGetY(tank) - missile->y;
    if ((d < 0) || (dx * dx + dy * dy < d)) {
      d = dx * dx + dy * dy;
      target = tank;
    }
  }
  return (target);
}

/*===========================================================================*/
/* ֥ȤȺ                                                  */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ֥Ȥ                                                        */
/*---------------------------------------------------------------------------*/

Missile MissileCreate(double x, double y, double vx, double vy, Tank tank,
		      int state, SubstanceList tank_list)
{
  Missile missile;

  missile = (Missile)malloc(sizeof(_Missile));
  if (missile == NULL) ErrorExit("Cannot create canon.");

  missile->x = x + vx * MISSILE_OFFSET;
  missile->y = y + vy * MISSILE_OFFSET;
  missile->vx = vx;
  missile->vy = vy;
  missile->tank = tank;
  missile->state = state;
  missile->target = MissileSearchTarget(missile, tank_list);

  TankIncMissile(tank);

  return (missile);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

Missile MissileDestroy(Missile missile)
{
  if (missile == NULL) return (NULL);

  TankDecMissile(missile->tank);
  free(missile);

  return (NULL);
}

/*===========================================================================*/
/* ư                                                                  */
/*===========================================================================*/

static int MissileMove(Missile missile, Map map)
{
  double dx, dy, d, vd;

  if (missile->target) {
    dx = TankGetX(missile->target) - missile->x;
    dy = TankGetY(missile->target) - missile->y;
    d = sqrt(dx*dx + dy*dy);
    if (d > 0.01) {
      missile->vx += dx * MISSILE_QUICK / d;
      missile->vy += dy * MISSILE_QUICK / d;
      vd = sqrt(missile->vx * missile->vx + missile->vy * missile->vy);
      if (vd > MISSILE_SPEED) {
	missile->vx *= MISSILE_SPEED / vd;
	missile->vy *= MISSILE_SPEED / vd;
      }
    }
  }

  missile->x += missile->vx;
  missile->y += missile->vy;

  return (0);
}

int MissileAct(Missile missile, Stage stage)
{
  switch (missile->state) {
  case MISSILE_WAIT :
    break;
  case MISSILE_ACTIVE :
    MissileMove(missile, StageGetMap(stage));
    MissileCheckExplosion(missile,
			  StageGetMap(stage),
			  StageGetTankList(stage),
			  StageGetExplosionList(stage));
    break;
  case MISSILE_EXPLOSION :
    break;
  case MISSILE_DESTROY :
    break;
  default :
    break;
  }
  return (missile->state);
}

/*===========================================================================*/
/* ǡ                                                            */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ѥǡκ                                                        */
/*---------------------------------------------------------------------------*/

char * MissileMakeDrawData(Missile missile, char * p)
{
  if (!missile) return (NULL);
  if (!p      ) return (NULL);

  if (missile->state != MISSILE_ACTIVE) return (p);

  sprintf(p, MISSILE_DRAW "\t%3.1f %3.1f\n", missile->x, missile->y);

  while (*(p++) != '\n') { /* None */ }
  *p = '\0';

  return (p);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
