/*****************************************************************************/
/* Socket.c - A library for network communication.                           */
/*                                                                           */
/* Socket.c Copyright (c) 2000 Sakai Hiroaki.                                */
/* All Rights Reserved.                                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Socket 饹(å̿ѤΥ饹)Υ饤֥                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "SocketP.h"

/*****************************************************************************/
/* ؿ(˸ʤ)                                          */
/*****************************************************************************/

/*===========================================================================*/
/* Socket ֥ȤΥåȤȥɥ쥹롥                     */
/* :0 (顼ˤ 1)                                                 */
/*===========================================================================*/

static int SocketMakeSocketAndAddress(Socket sockt)
{
  sockt->socket = socket(AF_INET, SOCK_STREAM, 0);
  if (sockt->socket == -1) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Cannot create socket.");
  }

  sockt->host = gethostbyname(sockt->hostname);
  if (sockt->host == NULL) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Fail gethostbyname().");
  }

  bzero((char *)(&(sockt->address)), sizeof(sockt->address));
  sockt->address.sin_family = AF_INET;
  /* sockt->address.sin_addr = ? See ftpd.c */
  sockt->address.sin_port = htons(sockt->port);
  bcopy(sockt->host->h_addr, &(sockt->address.sin_addr),
	sockt->host->h_length);

  return (0);
}

static int SocketBind(Socket sockt)
{
  if (bind(sockt->socket, (struct sockaddr *)(&(sockt->address)),
	   sizeof(sockt->address)) == -1) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Fail bind().");
  }

  return (0);
}

static int SocketListen(Socket sockt, int backlog)
{
  if (listen(sockt->socket, backlog) == -1) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Fail listen().");
  }

  return (0);
}

static int SocketConnect(Socket sockt)
{
  if (connect(sockt->socket, (struct sockaddr *)(&(sockt->address)),
	      sizeof(sockt->address)) == -1) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Fail connect().");
  }

  return (0);
}

/*****************************************************************************/
/* ؿ(˸)                                            */
/*****************************************************************************/

/*===========================================================================*/
/* accept() μ¹ԡ                                                         */
/*===========================================================================*/

int SocketAccept(Socket sockt)
{
  int len;

  len = sizeof(sockt->address);
  return (accept(sockt->socket, (struct sockaddr *)(&sockt->address), &len));
}

/*===========================================================================*/
/* åֹ֤                                                      */
/*===========================================================================*/

int SocketGetSocket(Socket this)
{
  return (this->socket);
}

/*===========================================================================*/
/* Socket ֥Ȥ롥                                         */
/*===========================================================================*/

Socket SocketCreate(char * hostname, unsigned short int port)
{
  Socket sockt;
  char buf[200];

  sockt = (Socket)malloc(sizeof(_Socket));
  if (sockt == NULL) ErrorExit("Cannot allocate memory for socket.");

  sockt->port = port;
  sockt->socket = 0;

  /* ۥ̾μ */
  if (!hostname) {
    gethostname(buf, 200);
    hostname = buf;
  }
  sockt->hostname = strdup(hostname);
  if (sockt->hostname == NULL) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Cannot copy hostname.");
  }

  if (SocketMakeSocketAndAddress(sockt)) {
    sockt = SocketDestroy(sockt);
    ErrorExit("Cannot make socket.");
  }

  return (sockt);
}

Socket SocketCreateToAccept(char * hostname, unsigned short int port,
			    int backlog)
{
  Socket s;

  s = SocketCreate(hostname, port);
  if (!s) {
    s = SocketDestroy(s);
    ErrorExit("Cannot create socket.");
  }

  if (SocketBind(s)) {
    s = SocketDestroy(s);
    ErrorExit("Cannot bind.");
  }

  if (SocketListen(s, backlog)) {
    s = SocketDestroy(s);
    ErrorExit("Cannot listen.");
  }

  return (s);
}

Socket SocketCreateToConnect(char * hostname, unsigned short int port)
{
  Socket s;

  s = SocketCreate(hostname, port);
  if (!s) {
    s = SocketDestroy(s);
    ErrorExit("Cannot create socket.");
  }

  if (SocketConnect(s)) {
    s = SocketDestroy(s);
    ErrorExit("Cannot connect.");
  }

  return (s);
}

/*===========================================================================*/
/* Socket ֥Ȥõ롥                                         */
/*===========================================================================*/

Socket SocketDestroy(Socket sockt)
{
  if (sockt == NULL) return (NULL);

  if (sockt->hostname != NULL) free(sockt->hostname);
  if (sockt->socket > 0) close(sockt->socket); /* ɤ close() 뤫 */

  free(sockt);

  return (NULL);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
