/*****************************************************************************/
/* Substance.c - A library for object list.                                  */
/*                                                                           */
/* Substance.c Copyright (c) 2000 Sakai Hiroaki.                             */
/* All Rights Reserved.                                                      */
/*****************************************************************************/

#include "SubstanceP.h"

/*****************************************************************************/
/* 饹                                                              */
/*****************************************************************************/

char * ClassGetName(SubstanceClass substance_class)
{
  return (substance_class->class_name);
}

/*****************************************************************************/
/* Substance                                                            */
/*****************************************************************************/

char * SubstanceGetClassName(Substance substance)
{ return ((*((SubstanceCore *)substance))->class_name); }
int SubstanceGetNumber(Substance substance)
{ return ((*((SubstanceCore *)substance))->number); }
int SubstanceSetNumber(Substance substance, int number)
{ return ((*((SubstanceCore *)substance))->number = number); }
int SubstanceGetSerial(Substance substance)
{ return ((*((SubstanceCore *)substance))->serial); }
int SubstanceSetSerial(Substance substance, int serial)
{ return ((*((SubstanceCore *)substance))->serial = serial); }
Substance SubstanceGetParent(Substance substance)
{ return ((*((SubstanceCore *)substance))->parent); }
SubstanceList SubstanceGetChildren(Substance substance)
{ return ((*((SubstanceCore *)substance))->children); }

int SubstanceIsClass(Substance substance, SubstanceClass substance_class)
{ return (!strcmp(SubstanceGetClassName(substance), substance_class->class_name)); }

/*****************************************************************************/
/* SubstanceCore                                                        */
/*****************************************************************************/

SubstanceCore SubstanceCoreCreate(SubstanceClass substance_class, Substance parent)
{
  SubstanceCore core;
  static int serial_number = 0; /* ƥ֥ˤĤ롤ꥢֹ */

  core = (SubstanceCore)(malloc(sizeof(_SubstanceCore)));
  if (!core) return (NULL);

  core->class_name = strdup(substance_class->class_name);
  core->number = (substance_class->number)++;
  core->serial = serial_number++;
  core->parent = parent;
  core->children = SubstanceListCreate();

  return (core);
}

SubstanceCore SubstanceCoreDestroy(SubstanceCore core)
{
  if (!core) return (NULL);
  if (core->class_name) free(core->class_name);
  SubstanceListDestroy(core->children);
  free(core);
  return (NULL);
}

/*****************************************************************************/
/* SubstanceListData                                                    */
/*****************************************************************************/

static SubstanceListData SubstanceListDataCreate(Substance substance,
					   Substance (*destructor)())
{
  SubstanceListData list_data;

  list_data = (SubstanceListData)malloc(sizeof(_SubstanceListData));

  list_data->substance = substance;
  list_data->destructor = destructor;

  return (list_data);
}

static SubstanceListData SubstanceListDataDestroy(SubstanceListData list_data)
{
  if (list_data == NULL) return (NULL);

  /* ǥȥ饯μ¹ */
  if (list_data->destructor)
    (*(list_data->destructor))(list_data->substance);

  free(list_data);

  return (NULL);
}

/*****************************************************************************/
/* SubstanceList ֥Ȥ                                           */
/*****************************************************************************/

Substance SubstanceListGetSubstance(SubstanceList list)
{
  if (list == NULL) return (NULL);
  return (list->current->substance);
}

int SubstanceListGetLength(SubstanceList list)
{
  if (list == NULL) return (-1);
  return (list->length);
}

int SubstanceListIsEmpty(SubstanceList list)
{
  if (list == NULL) return (1);
  return (list->start_edge->next == list->end_edge);
}

int SubstanceListIsCurrentStartEdge(SubstanceList list)
{
  if (list == NULL) return (0);
  return (list->current == list->start_edge);
}

int SubstanceListIsCurrentEndEdge(SubstanceList list)
{
  if (list == NULL) return (0);
  return (list->current == list->end_edge);
}

int SubstanceListIsCurrentStart(SubstanceList list)
{
  if (list == NULL) return (0);
  return (list->current == list->start_edge->next);
}

int SubstanceListIsCurrentEnd(SubstanceList list)
{
  if (list == NULL) return (0);
  return (list->current == list->start_edge->prev);
}

int SubstanceListSetCurrentToStartEdge(SubstanceList list)
{
  if (list == NULL) return (1);
  list->current = list->start_edge;
  return (0);
}

int SubstanceListSetCurrentToEndEdge(SubstanceList list)
{
  if (list == NULL) return (1);
  list->current = list->end_edge;
  return (0);
}

int SubstanceListSetCurrentToStart(SubstanceList list)
{
  if (list == NULL) return (1);
  list->current = list->start_edge->next;
  return (0);
}

int SubstanceListSetCurrentToEnd(SubstanceList list)
{
  if (list == NULL) return (1);
  list->current = list->end_edge->prev;
  return (0);
}

int SubstanceListMoveCurrentToPrev(SubstanceList list)
{
  if (list == NULL) return (1);
  if (list->current->prev == NULL) return (1);
  list->current = list->current->prev;
  return (0);
}

int SubstanceListMoveCurrentToNext(SubstanceList list)
{
  if (list == NULL) return (1);
  if (list->current->next == NULL) return (1);
  list->current = list->current->next;
  return (0);
}

int SubstanceListInsertSubstanceToPrev(SubstanceList list, Substance substance,
				 Substance (*destructor)())
{
  SubstanceListData list_data;

  if (list == NULL) return (1);
  if (SubstanceListIsCurrentStartEdge(list)) return (1);

  list_data = SubstanceListDataCreate(substance, destructor);
  if (list_data == NULL) return (1);

  list_data->prev = list->current->prev;
  list_data->next = list->current;
  list->current->prev->next = list_data;
  list->current->prev = list_data;
  list->current = list_data;
  (list->length)++;

  return (0);
}

int SubstanceListInsertSubstanceToNext(SubstanceList list, Substance substance,
				 Substance (*destructor)())
{
  SubstanceListData list_data;

  if (list == NULL) return (1);
  if (SubstanceListIsCurrentEndEdge(list)) return (1);

  list_data = SubstanceListDataCreate(substance, destructor);
  if (list_data == NULL) return (1);

  list_data->next = list->current->next;
  list_data->prev = list->current;
  list->current->next->prev = list_data;
  list->current->next = list_data;
  list->current = list_data;
  (list->length)++;

  return (0);
}

int SubstanceListInsertSubstanceToStart(SubstanceList list, Substance substance,
				  Substance (*destructor)())
{
  SubstanceListSetCurrentToStart(list);
  return (SubstanceListInsertSubstanceToPrev(list, substance, destructor));
}

int SubstanceListInsertSubstanceToEnd(SubstanceList list, Substance substance,
				Substance (*destructor)())
{
  SubstanceListSetCurrentToEnd(list);
  return (SubstanceListInsertSubstanceToNext(list, substance, destructor));
}

int SubstanceListDeleteSubstanceToPrev(SubstanceList list)
{
  SubstanceListData current;

  if (list == NULL) return (1);
  if (SubstanceListIsCurrentStartEdge(list) || SubstanceListIsCurrentEndEdge(list))
    return (1);

  list->current->prev->next = list->current->next;
  list->current->next->prev = list->current->prev;

  current = list->current;
  list->current = list->current->prev;

  current = SubstanceListDataDestroy(current);
  (list->length)--;

  return (0);
}

int SubstanceListDeleteSubstanceToNext(SubstanceList list)
{
  SubstanceListData current;

  if (list == NULL) return (1);
  if (SubstanceListIsCurrentStartEdge(list) || SubstanceListIsCurrentEndEdge(list))
    return (1);

  list->current->prev->next = list->current->next;
  list->current->next->prev = list->current->prev;

  current = list->current;
  list->current = list->current->next;

  current = SubstanceListDataDestroy(current);
  (list->length)--;

  return (0);
}

int SubstanceListDeleteSubstanceFromStart(SubstanceList list)
{
  if (list == NULL) return (1);
  SubstanceListSetCurrentToStart(list);
  return (SubstanceListDeleteSubstanceToNext(list));
}

int SubstanceListDeleteSubstanceFromEnd(SubstanceList list)
{
  if (list == NULL) return (1);
  SubstanceListSetCurrentToEnd(list);
  return (SubstanceListDeleteSubstanceToPrev(list));
}

SubstanceList SubstanceListCreate() /* SubstanceList ֥Ȥ */
{
  SubstanceList list;

  list = (SubstanceList)malloc(sizeof(_SubstanceList));
  if (list == NULL) return (NULL);

  list->start_edge   = SubstanceListDataCreate(NULL, NULL);
  list->end_edge     = SubstanceListDataCreate(NULL, NULL);
  list->current = list->start_edge;
  list->length  = 0;    /* ¸ߤƤǡο */

  list->start_edge->prev = NULL;
  list->start_edge->next = list->end_edge;
  list->end_edge->prev   = list->start_edge;
  list->end_edge->next   = NULL;

  return (list);
}

SubstanceList SubstanceListDestroy(SubstanceList list) /*  */
{
  if (list == NULL) return (NULL);

  while (!SubstanceListIsEmpty(list))
    SubstanceListDeleteSubstanceFromStart(list);

  if (list->start_edge)
    list->start_edge = SubstanceListDataDestroy(list->start_edge);
  if (list->end_edge)
    list->end_edge = SubstanceListDataDestroy(list->end_edge);

  free(list);

  return (NULL);
}

Substance SubstanceListSearchSubstance(SubstanceList list, int serial)
{
  for (SubstanceListSetCurrentToStart(list);
       !SubstanceListIsCurrentEndEdge(list);
       SubstanceListMoveCurrentToNext(list)) {

    if (SubstanceGetSerial(SubstanceListGetSubstance(list)) == serial)
      return (SubstanceListGetSubstance(list));
  }

  return (NULL);
}

/* End of File. */
